from . import ACProtocol


class Daikin(ACProtocol):
    # Daikin timing constants
    HDR_MARK = 5050
    HDR_SPACE = 2100
    BIT_MARK = 391
    ONE_SPACE = 1725
    ZERO_SPACE = 667
    MSG_SPACE = 30000

    # Daikin codes
    DAIKIN_MODE_AUTO = 0x10  # Operating mode
    DAIKIN_MODE_HEAT = 0x40
    DAIKIN_MODE_COOL = 0x30
    DAIKIN_MODE_DRY = 0x20
    DAIKIN_MODE_FAN = 0x60

    DAIKIN_FAN_AUTO = 0x0A  # Fan speed
    DAIKIN_FAN1 = 0x03
    DAIKIN_FAN2 = 0x04
    DAIKIN_FAN3 = 0x05
    DAIKIN_FAN4 = 0x06
    DAIKIN_FAN5 = 0x07

    op_modes = {
        ACProtocol.MODE_AUTO: DAIKIN_MODE_AUTO,
        ACProtocol.MODE_HEAT: DAIKIN_MODE_HEAT,
        ACProtocol.MODE_COOL: DAIKIN_MODE_COOL,
        ACProtocol.MODE_DRY: DAIKIN_MODE_DRY,
        ACProtocol.MODE_FAN: DAIKIN_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: DAIKIN_FAN_AUTO,
        ACProtocol.FAN_1: DAIKIN_FAN1,
        ACProtocol.FAN_2: DAIKIN_FAN2,
        ACProtocol.FAN_3: DAIKIN_FAN3,
        ACProtocol.FAN_4: DAIKIN_FAN4,
        ACProtocol.FAN_5: DAIKIN_FAN5,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        power_mode = 0x1 if power_mode_cmd == self.POWER_ON else 0x0
        op_mode = self.op_modes.get(operating_mode_cmd, self.DAIKIN_MODE_AUTO)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.DAIKIN_FAN_AUTO)
        temperature = 0x10  # 18 deg
        if (operating_mode_cmd == self.MODE_HEAT and 13 < temperature_cmd < 29) \
                or (operating_mode_cmd == self.MODE_COOL and 17 < temperature_cmd < 33) \
                or 17 < temperature_cmd < 30:
            temperature = (temperature_cmd << 1) - 20
        if operating_mode_cmd == self.MODE_DRY:
            temperature = 0x80
        if operating_mode_cmd == self.MODE_FAN:
            temperature = 0xc0

        self.send_daikin(power_mode, op_mode, fan_speed, temperature)

    def send_daikin(self, power_mode, op_mode, fan_speed, temperature):
        send_buffer = bytearray(b'\x11\xDA\x27\xF0\x0D\x00\x0F\x11\xDA\x27\x00\xD3\x11\x00\x00\x00\x1E\x0A\x08\x26')
        send_buffer[12] = power_mode | op_mode
        send_buffer[16] = temperature
        send_buffer[17] = fan_speed

        checksum = 0
        for i in range(7, 19):
            checksum = checksum + send_buffer[i]

        send_buffer[19] = checksum & 0xff

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(0, 7):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(self.MSG_SPACE)

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(7, 20):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(0)




from . import ACProtocol


class Mitsubishi(ACProtocol):
    # Midea timing constants
    HDR_MARK = 3500
    HDR_SPACE = 1700
    BIT_MARK = 430
    ONE_SPACE = 1250
    ZERO_SPACE = 390
    MSG_SPACE = 17500

    # MIDEA codes

    MITSUBISHI_AIRCON1_MODE_AUTO = 0x20
    MITSUBISHI_AIRCON3_MODE_AUTO = 0x60  # FA auto mode
    MITSUBISHI_AIRCON1_MODE_HEAT = 0x08
    MITSUBISHI_AIRCON3_MODE_HEAT = 0x48  # FA heat mode
    MITSUBISHI_AIRCON1_MODE_COOL = 0x18
    MITSUBISHI_AIRCON2_MODE_COOL = 0x18  # MSY cool mode
    MITSUBISHI_AIRCON3_MODE_COOL = 0x58  # FA cool mode
    MITSUBISHI_AIRCON1_MODE_DRY = 0x10
    MITSUBISHI_AIRCON2_MODE_DRY = 0x18  # MSY DRY mode
    MITSUBISHI_AIRCON3_MODE_DRY = 0x50  # FA dry mode
    MITSUBISHI_AIRCON1_MODE_FAN = 0x38  # EF 'FAN' mode
    MITSUBISHI_AIRCON2_MODE_FAN = 0x38  # MSY fan mode
    MITSUBISHI_AIRCON1_MODE_ISEE = 0x40  # Isee
    MITSUBISHI_AIRCON2_MODE_IFEEL = 0x00  # MSY

    MITSUBISHI_AIRCON1_MODE_OFF = 0x00  # Power OFF
    MITSUBISHI_AIRCON1_MODE_ON = 0x20  # Power ON

    temperatures = [0, 8, 12, 4, 6, 14, 10, 2, 3, 11, 9, 1, 5, 13]

    op_modes = {
        ACProtocol.MODE_AUTO: MIDEA_AIRCON1_MODE_AUTO,
        ACProtocol.MODE_HEAT: MIDEA_AIRCON1_MODE_HEAT,
        ACProtocol.MODE_COOL: MIDEA_AIRCON1_MODE_COOL,
        ACProtocol.MODE_DRY: MIDEA_AIRCON1_MODE_DRY,
        ACProtocol.MODE_FAN: MIDEA_AIRCON1_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: MIDEA_AIRCON1_FAN_AUTO,
        ACProtocol.FAN_1: MIDEA_AIRCON1_FAN1,
        ACProtocol.FAN_2: MIDEA_AIRCON1_FAN2,
        ACProtocol.FAN_3: MIDEA_AIRCON1_FAN3,
    }

    def __init__(self):
        ACProtocol.__init__(self)

    @classmethod
    def list_modes(cls):
        return list(cls.op_modes)

    @classmethod
    def list_fan_speeds(cls):
        return list(cls.fan_speeds)

    # noinspection PyUnusedLocal
    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        operating_mode = self.MIDEA_AIRCON1_MODE_OFF if power_mode_cmd == self.POWER_OFF \
            else self.op_modes.get(operating_mode_cmd, self.MIDEA_AIRCON1_MODE_HEAT)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.MIDEA_AIRCON1_FAN_AUTO)
        temperature = 23
        if 16 < temperature_cmd < 31:
            temperature = self.temperatures[(temperature_cmd - 17)]
        self.durations = []
        self.send_midea(operating_mode, fan_speed, temperature)

    def send_midea(self, operating_mode, fan_speed, temperature):
        send_buffer = bytearray(b'\x4d\xde\x07')  # Turn OFF default value
        if operating_mode != self.MIDEA_AIRCON1_MODE_OFF:
            send_buffer[1] = ~fan_speed & 0xFF
            if operating_mode == self.MIDEA_AIRCON1_MODE_FAN:
                send_buffer[2] = self.MIDEA_AIRCON1_MODE_DRY | 0x07
            else:
                send_buffer[2] = operating_mode | temperature
        return self.send_midea_raw(send_buffer)

    def send_midea_raw(self, send_buffer):

        self.durations.extend([self.HDR_MARK, self.HDR_SPACE, ])
        for i in range(3):
            self.send_byte(send_buffer[i])
            self.send_byte(~send_buffer[i])
        self.mark()
        self.space(self.MSG_SPACE)
        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(3):
            self.send_byte(send_buffer[i])
            self.send_byte(~send_buffer[i])
        self.mark()
        self.space(0)



from importlib import import_module


class ACProtocol:
    HDR_MARK = 4420
    HDR_SPACE = 4300
    BIT_MARK = 620
    ONE_SPACE = 1560
    ZERO_SPACE = 480
    MSG_SPACE = 5100

    # Commands
    POWER_OFF = 'off'
    POWER_ON = 'on'

    # Operating modes
    MODE_AUTO = 'auto'
    MODE_HEAT = 'heat'
    MODE_COOL = 'cool'
    MODE_DRY = 'dry'
    MODE_FAN = 'fan'

    # Fan speeds. Note that some heatpumps have less than 5 fan speeds
    FAN_AUTO = 'auto'
    FAN_1 = 'low'
    FAN_2 = 'medium'
    FAN_3 = 'high'
    FAN_4 = 'higher'
    FAN_5 = 'highest'

    # Vertical air directions. Note that these cannot be set on all heat pumps
    VDIR_AUTO = 'auto'
    VDIR_MANUAL = 'manual'
    VDIR_UP = 'up'
    VDIR_MUP = 'middle_up'
    VDIR_MIDDLE = 'middle'
    VDIR_MDOWN = 'middle_down'
    VDIR_DOWN = 'down'
    VDIR_SWING = 'swing'
    VDIR_SWING_UP = 'swing_up'
    VDIR_SWING_MIDDLE = 'swing_middle'
    VDIR_SWING_DOWN = 'swing_down'

    # Horizontal air directions. Note that these cannot be set on all heat pumps
    HDIR_AUTO = 'auto'
    HDIR_MANUAL = 'manual'
    HDIR_MIDDLE = 'middle'
    HDIR_LEFT = 'left'
    HDIR_MLEFT = 'middle_left'
    HDIR_MRIGHT = 'middle_right'
    HDIR_RIGHT = 'right'
    HDIR_WIDE = 'wide'
    HDIR_SWING = 'swing'
    HDIR_SWING_SPREAD = 'swing_spread'

    def __init__(self):
        self.durations = []

    @classmethod
    def list_modes(cls):
        return list()

    @classmethod
    def list_fan_speeds(cls):
        return list()

    @classmethod
    def list_swing_modes(cls):
        return list()

    def get_durations(self):
        return self.durations

    @classmethod
    def is_swing(cls, vdir, hdir=HDIR_AUTO):
        return vdir in [cls.VDIR_SWING, cls.VDIR_SWING_DOWN, cls.VDIR_SWING_MIDDLE, cls.VDIR_SWING_UP] or \
               hdir in [cls.HDIR_SWING, cls.HDIR_SWING_SPREAD]

    def send_byte(self, val):
        for i in range(8):
            self.bit(val)
            val = val >> 1

    def mark(self, duration=-1):
        if duration == -1:
            duration = self.BIT_MARK
        self.durations.append(duration)

    def space(self, duration=-1):
        if duration == -1:
            duration = self.ZERO_SPACE
        self.durations.append(duration)

    def bit(self, val):
        self.durations.append(self.BIT_MARK)
        if val & 0x01:
            self.durations.append(self.ONE_SPACE)
        else:
            self.durations.append(self.ZERO_SPACE)

    @staticmethod
    def bit_reverse(val):
        x = val
        #          01010101  |         10101010
        x = ((x >> 1) & 0x55) | ((x << 1) & 0xaa)
        #          00110011  |         11001100
        x = ((x >> 2) & 0x33) | ((x << 2) & 0xcc)
        #          00001111  |         11110000
        x = ((x >> 4) & 0x0f) | ((x << 4) & 0xf0)
        return x

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_mode=False):
        pass


def get_sender(ac_type):
    try:
        return getattr(import_module('hvac_ir.' + ac_type.lower()), ac_type.title())
    except ImportError:
        return None



#import broadlink
# import argparse
# import time
import binascii
# import sys
from . import ACProtocol
import logging

_LOGGER = logging.getLogger(__name__)


class Gree(ACProtocol):
    HDR_MARK = 9000
    HDR_SPACE = 4000
    BIT_MARK = 620
    ONE_SPACE = 1600
    ZERO_SPACE = 540
    MSG_SPACE = 19000

    GREE_POWER_OFF = 0x00
    GREE_POWER_ON = 0x08

    # Operating modes
    GREE_MODE_AUTO = 0x00
    GREE_MODE_HEAT = 0x04
    GREE_MODE_COOL = 0x01
    GREE_MODE_DRY = 0x02
    GREE_MODE_FAN = 0x03

    # Fan speeds
    GREE_FAN_AUTO = 0x00
    GREE_FAN1 = 0x10
    GREE_FAN2 = 0x20
    GREE_FAN3 = 0x30

    # Only  available on YAN Vertical air directions
    GREE_VDIR_AUTO = 0x00
    GREE_VDIR_MANUAL = 0x00
    GREE_VDIR_SWING = 0x01
    GREE_VDIR_SWING_UP = 0xB
    GREE_VDIR_SWING_MIDDLE = 0x9
    GREE_VDIR_SWING_DOWN = 0x7
    GREE_VDIR_UP = 0x02
    GREE_VDIR_MUP = 0x03
    GREE_VDIR_MIDDLE = 0x04
    GREE_VDIR_MDOWN = 0x05
    GREE_VDIR_DOWN = 0x06

    GREE_HDIR_AUTO = 0
    GREE_HDIR_LEFT = 0x20
    GREE_HDIR_MLEFT = 0x30
    GREE_HDIR_MIDDLE = 0x40
    GREE_HDIR_MRIGHT = 0x50
    GREE_HDIR_RIGHT = 0x60
    GREE_HDIR_WIDE = 0xC0
    GREE_HDIR_SWING = 0x10
    GREE_HDIR_SWING_SPREAD = 0xD0
    MODEL = False

    op_modes = {
        ACProtocol.MODE_AUTO: GREE_MODE_AUTO,
        ACProtocol.MODE_HEAT: GREE_MODE_HEAT,
        ACProtocol.MODE_COOL: GREE_MODE_COOL,
        ACProtocol.MODE_DRY: GREE_MODE_DRY,
        ACProtocol.MODE_FAN: GREE_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: GREE_FAN_AUTO,
        ACProtocol.FAN_1: GREE_FAN1,
        ACProtocol.FAN_2: GREE_FAN2,
        ACProtocol.FAN_3: GREE_FAN3,
    }

    swing_v_map = {
        ACProtocol.VDIR_AUTO: GREE_VDIR_AUTO,
        ACProtocol.VDIR_MANUAL: GREE_VDIR_MANUAL,
        ACProtocol.VDIR_UP: GREE_VDIR_UP,
        ACProtocol.VDIR_MUP: GREE_VDIR_MUP,
        ACProtocol.VDIR_MIDDLE: GREE_VDIR_MIDDLE,
        ACProtocol.VDIR_MDOWN: GREE_VDIR_MDOWN,
        ACProtocol.VDIR_DOWN: GREE_VDIR_DOWN,
        ACProtocol.VDIR_SWING: GREE_VDIR_SWING,
        ACProtocol.VDIR_SWING_UP: GREE_VDIR_SWING_UP,
        ACProtocol.VDIR_SWING_MIDDLE: GREE_VDIR_SWING_MIDDLE,
        ACProtocol.VDIR_SWING_DOWN: GREE_VDIR_SWING_DOWN,
    }

    swing_h_map = {
        ACProtocol.HDIR_AUTO: GREE_HDIR_AUTO,
        ACProtocol.HDIR_MANUAL: GREE_HDIR_AUTO,
        ACProtocol.HDIR_MIDDLE: GREE_HDIR_MIDDLE,
        ACProtocol.HDIR_LEFT: GREE_HDIR_LEFT,
        ACProtocol.HDIR_MLEFT: GREE_HDIR_MLEFT,
        ACProtocol.HDIR_MRIGHT: GREE_HDIR_MRIGHT,
        ACProtocol.HDIR_RIGHT: GREE_HDIR_RIGHT,
        ACProtocol.HDIR_WIDE: GREE_HDIR_WIDE,
        ACProtocol.HDIR_SWING: GREE_HDIR_SWING,
        ACProtocol.HDIR_SWING_SPREAD: GREE_HDIR_SWING_SPREAD,
    }

    def __init__(self):
        ACProtocol.__init__(self)

    @classmethod
    def list_modes(cls):
        return list(cls.op_modes)

    @classmethod
    def list_fan_speeds(cls):
        return list(cls.fan_speeds)

    @classmethod
    def list_swing_modes(cls):
        return list(cls.swing_v_map)

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_mode=False):
        power_mode = self.GREE_POWER_ON if power_mode_cmd == self.POWER_ON else self.GREE_POWER_OFF
        operating_mode = self.op_modes.get(operating_mode_cmd, self.GREE_MODE_HEAT)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.GREE_FAN_AUTO)
        swing = self.is_swing(swing_v_cmd, swing_h_cmd)
        swing_v = self.swing_v_map.get(swing_v_cmd, self.GREE_VDIR_AUTO)
        swing_h = self.swing_h_map.get(swing_h_cmd, self.GREE_HDIR_AUTO)
        temperature = 23
        if 15 < temperature_cmd < 31:
            temperature = temperature_cmd - 16
        self.durations = []
        self.send_gree(power_mode, operating_mode, fan_speed, temperature, swing, swing_v, swing_h,
                       turbo_mode)

    @staticmethod
    def make_data(power_mode, operating_mode, fan_speed, temperature, swing, swing_v, swing_h, turbo_mode=False):
        data = bytearray(8)
        swing_flag = 0x40 if swing else 0

        data[0] = fan_speed | operating_mode | power_mode | swing_flag
        data[1] = temperature
        data[2] = 0x70 if turbo_mode else 0x60
        data[3] = 0x50
        data[4] = swing_v | swing_h
        data[5] = 0x40
        data[6] = 0
        data[7] = (((data[0] & 0x0F) +
                    (data[1] & 0x0F) +
                    (data[2] & 0x0F) +
                    (data[3] & 0x0F) +
                    ((data[4] & 0xF0) >> 4) +
                    ((data[5] & 0xF0) >> 4) +
                    ((data[6] & 0xF0) >> 4) +
                    0x0A) & 0x0F) << 4
        return data

    def send_gree(self, power_mode, operating_mode, fan_speed, temperature, swing, swing_v, swing_h, turbo_mode=False):
        data = self.make_data(power_mode, operating_mode, fan_speed, temperature, swing, swing_v, swing_h, turbo_mode)

        self.send_train(data)

        data[3] = 0x70
        data[4] = 0
        data[5] = 0
        data[6] = 0
        data[7] = (((data[0] & 0x0F) +
                    (data[1] & 0x0F) +
                    (data[2] & 0x0F) +
                    (data[3] & 0x0F) +
                    ((data[4] & 0xF0) >> 4) +
                    ((data[5] & 0xF0) >> 4) +
                    ((data[6] & 0xF0) >> 4) +
                    0x0A) & 0x0F) << 4

        self.send_train(data)

    def send_train(self, data):
        for c in data:
            _LOGGER.debug("%02x" % c)
        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)
        for i in range(4):
            self.send_byte(data[i])
        # send what's left of byte 4
        self.bit(0)
        self.bit(1)
        self.bit(0)
        self.mark()
        self.space(self.MSG_SPACE)
        for i in range(4, 8):
            self.send_byte(data[i])
        self.mark()
        self.space(self.MSG_SPACE)



from . import ACProtocol


class Ballu(ACProtocol):
    HDR_MARK = 9300  
    HDR_SPACE = 4550  
    BIT_MARK = 500   
    ZERO_SPACE = 500   
    ONE_SPACE = 1650  
    MSG_SPACE = 0     

    # BALLU codes
    BALLU_AIRCON_MODE_HEAT = 0x00
    BALLU_AIRCON_MODE_COOL = 0x02
    BALLU_AIRCON_MODE_DRY = 0x03
    BALLU_AIRCON_MODE_FAN = 0x04

    BALLU_AIRCON_MODE_OFF = 0x04 
    BALLU_AIRCON_MODE_ON = 0x00

    BALLU_AIRCON_FAN_AUTO = 0x00 
    BALLU_AIRCON_FAN1 = 0x01 
    BALLU_AIRCON_FAN2 = 0x02
    BALLU_AIRCON_FAN3 = 0x03 

    op_modes = {
        ACProtocol.MODE_HEAT: BALLU_AIRCON_MODE_HEAT,
        ACProtocol.MODE_COOL: BALLU_AIRCON_MODE_COOL,
        ACProtocol.MODE_DRY: BALLU_AIRCON_MODE_DRY,
        ACProtocol.MODE_FAN: BALLU_AIRCON_MODE_FAN,
    }
    
    fan_speeds = {
        ACProtocol.FAN_AUTO: BALLU_AIRCON_FAN_AUTO,
        ACProtocol.FAN_1: BALLU_AIRCON_FAN1,
        ACProtocol.FAN_2: BALLU_AIRCON_FAN2,
        ACProtocol.FAN_3: BALLU_AIRCON_FAN3,
    }

    def __init__(self):
        ACProtocol.__init__(self)

    @classmethod
    def list_modes(cls):
        return list(cls.op_modes)

    @classmethod
    def list_fan_speeds(cls):
        return list(cls.fan_speeds)

    # noinspection PyUnusedLocal
    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        power_mode = self.BALLU_AIRCON_MODE_OFF if power_mode_cmd == self.POWER_OFF else self.BALLU_AIRCON_MODE_ON
        operating_mode = self.op_modes.get(operating_mode_cmd, self.BALLU_AIRCON_MODE_COOL)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.BALLU_AIRCON_FAN_AUTO)
        temperature = 8
        if 15 < temperature_cmd < 31:
            temperature = temperature_cmd - 16
        self.durations = []
        self.send_ballu(power_mode, operating_mode, fan_speed, temperature)

    def send_ballu(self, power_mode,operating_mode, fan_speed, temperature):
        send_buffer = bytearray(b'\x83\x06\x04\x42\x00\x00')
        send_buffer[2] = power_mode if power_mode == self.BALLU_AIRCON_MODE_OFF else fan_speed
        send_buffer[3] = (temperature << 4) | operating_mode

        self.durations.extend([self.HDR_MARK, self.HDR_SPACE, ])
        for i in range(len(send_buffer)):
            self.send_byte(send_buffer[i])
        self.mark()
        self.space(0)



from . import ACProtocol


class Hitachi(ACProtocol):
    # Hitachi remote timing constants
    HDR_MARK = 3436
    HDR_SPACE = 1640
    BIT_MARK = 420
    ONE_SPACE = 1250
    ZERO_SPACE = 500

    # Hitachi codes
    HITACHI_MODE_AUTO = 0x02  # Operating mode
    HITACHI_MODE_HEAT = 0x03
    HITACHI_MODE_COOL = 0x04
    HITACHI_MODE_DRY = 0x05
    HITACHI_MODE_FAN = 0x0C

    HITACHI_POWER_OFF = 0x00  # Power OFF
    HITACHI_POWER_ON = 0x80

    HITACHI_FAN_AUTO = 0x01  # Fan speed
    HITACHI_FAN1 = 0x02
    HITACHI_FAN2 = 0x03
    HITACHI_FAN3 = 0x04
    HITACHI_FAN4 = 0x05

    HITACHI_VDIR_AUTO = 0x00
    HITACHI_VDIR_SWING = 0x01

    HITACHI_HDIR_AUTO = 0x00
    HITACHI_HDIR_SWING = 0x01

    op_modes = {
        ACProtocol.MODE_AUTO: HITACHI_MODE_AUTO,
        ACProtocol.MODE_HEAT: HITACHI_MODE_HEAT,
        ACProtocol.MODE_COOL: HITACHI_MODE_COOL,
        ACProtocol.MODE_DRY: HITACHI_MODE_DRY,
        ACProtocol.MODE_FAN: HITACHI_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: HITACHI_FAN_AUTO,
        ACProtocol.FAN_1: HITACHI_FAN1,
        ACProtocol.FAN_2: HITACHI_FAN2,
        ACProtocol.FAN_3: HITACHI_FAN3,
        ACProtocol.FAN_4: HITACHI_FAN4,
    }

    swing_v_map = {
        ACProtocol.VDIR_AUTO: HITACHI_VDIR_AUTO,
        ACProtocol.VDIR_SWING: HITACHI_VDIR_SWING,
    }

    swing_h_map = {
        ACProtocol.VDIR_AUTO: HITACHI_HDIR_AUTO,
        ACProtocol.VDIR_SWING: HITACHI_HDIR_SWING,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        op_mode = self.op_modes.get(operating_mode_cmd, self.HITACHI_MODE_HEAT)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.HITACHI_FAN_AUTO)
        swing_v = self.swing_v_map.get(swing_v_cmd, self.HITACHI_VDIR_AUTO)
        swing_h = self.swing_v_map.get(swing_h_cmd, self.HITACHI_HDIR_AUTO)

        temperature = 23
        if 15 < temperature_cmd < 33:
            temperature = temperature_cmd
        if op_mode == self.HITACHI_MODE_FAN:
            temperature = 64
            if fan_speed == self.HITACHI_FAN_AUTO:
                fan_speed = self.HITACHI_FAN2

        if op_mode == self.HITACHI_MODE_DRY:
            fan_speed = self.HITACHI_FAN2

        power_mode = self.HITACHI_POWER_ON if power_mode_cmd == self.POWER_ON else self.HITACHI_POWER_OFF

        self.send_hitachi(power_mode, op_mode, fan_speed, temperature, swing_v, swing_h)

    def send_hitachi(self, power_mode, op_mode, fan_speed, temperature, swing_v, swing_h):
        send_buffer = bytearray([0x01, 0x10, 0x30, 0x40, 0xBF, 0x01, 0xFE, 0x11, 0x12, 0x08, 0x00, 0x00, 0x00, 0x00,
                                 0x06, 0x06, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x00])
        if temperature == 16:
            send_buffer[9] = 0x09
        send_buffer[10] = op_mode
        send_buffer[11] = temperature << 1
        send_buffer[13] = fan_speed
        send_buffer[14] = send_buffer[14] | swing_v
        send_buffer[15] = send_buffer[15] | swing_h
        send_buffer[17] = power_mode

        checksum = 1086
        for i in range(0, 27):
            checksum = checksum - send_buffer[i]

        send_buffer[27] = checksum & 0xff

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(0, len(send_buffer)):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(0)



from . import ACProtocol


class Hyundai(ACProtocol):
    # Hyundai heatpump control (remote control P/N Y512F2)
    # Hyundai timing constants
    HDR_MARK = 8840  # 8700
    HDR_SPACE = 4440  # 4200
    BIT_MARK = 640  # 580
    ONE_SPACE = 1670  # 1530
    ZERO_SPACE = 570  # 460

    # Hyundai codes
    HYUNDAI_MODE_AUTO = 0x00  # Operating mode
    HYUNDAI_MODE_HEAT = 0x04
    HYUNDAI_MODE_COOL = 0x01
    HYUNDAI_MODE_DRY = 0x02
    HYUNDAI_MODE_FAN = 0x03
    HYUNDAI_POWER_OFF = 0x00  # Power OFF
    HYUNDAI_POWER_ON = 0x08  # Power ON
    HYUNDAI_FAN_AUTO = 0x00  # Fan speed
    HYUNDAI_FAN1 = 0x10
    HYUNDAI_FAN2 = 0x20
    HYUNDAI_FAN3 = 0x30

    HYUNDAI_VDIR_SWING = 0x40  # Vertical swing
    HYUNDAI_VDIR_AUTO = 0x00  # Automatic setting

    op_modes = {
        ACProtocol.MODE_AUTO: HYUNDAI_MODE_AUTO,
        ACProtocol.MODE_HEAT: HYUNDAI_MODE_HEAT,
        ACProtocol.MODE_COOL: HYUNDAI_MODE_COOL,
        ACProtocol.MODE_DRY: HYUNDAI_MODE_DRY,
        ACProtocol.MODE_FAN: HYUNDAI_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: HYUNDAI_FAN_AUTO,
        ACProtocol.FAN_1: HYUNDAI_FAN1,
        ACProtocol.FAN_2: HYUNDAI_FAN2,
        ACProtocol.FAN_3: HYUNDAI_FAN3,
    }

    swing_v_map = {
        ACProtocol.VDIR_AUTO: HYUNDAI_VDIR_AUTO,
        ACProtocol.VDIR_SWING: HYUNDAI_VDIR_SWING,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        op_mode = self.op_modes.get(operating_mode_cmd, self.HYUNDAI_MODE_HEAT)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.HYUNDAI_FAN_AUTO)
        swing_v = self.swing_v_map.get(swing_v_cmd, self.HYUNDAI_VDIR_AUTO)

        temperature = 23
        if 15 < temperature_cmd < 31:
            temperature = temperature_cmd
        if op_mode == self.HYUNDAI_MODE_FAN:
            temperature = 24
            if fan_speed == self.HYUNDAI_FAN_AUTO:
                fan_speed = self.HYUNDAI_FAN2

        power_mode = self.HYUNDAI_POWER_ON if power_mode_cmd == self.POWER_ON else self.HYUNDAI_POWER_OFF

        self.send_hyundai(power_mode, op_mode, fan_speed, temperature, swing_v)

    def send_hyundai(self, power_mode, op_mode, fan_speed, temperature, swing_v):
        send_buffer = bytearray([0x00, 0x00, 0x00, 0x50])
        send_buffer[0] = op_mode | fan_speed | power_mode | swing_v
        send_buffer[1] = temperature - 16

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(0, len(send_buffer)):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(self.ZERO_SPACE)
        self.mark()
        self.space(self.ONE_SPACE)
        self.mark()
        self.space(self.ZERO_SPACE)

        self.mark()
        self.space(0)



from . import ACProtocol


class Fuego(ACProtocol):
    # Fuego timing constants
    HDR_MARK = 3600
    HDR_SPACE = 1630
    BIT_MARK = 420
    ONE_SPACE = 1380
    ZERO_SPACE = 420

    # Fuego codes
    FUEGO_MODE_AUTO = 0x08  # Operating mode
    FUEGO_MODE_HEAT = 0x01
    FUEGO_MODE_COOL = 0x03
    FUEGO_MODE_DRY = 0x02
    FUEGO_MODE_FAN = 0x07
    FUEGO_MODE_ON = 0x04  # Power ON
    FUEGO_MODE_OFF = 0x00
    FUEGO_FAN_AUTO = 0x00  # Fan speed
    FUEGO_FAN1 = 0x02
    FUEGO_FAN2 = 0x03
    FUEGO_FAN3 = 0x05

    FUEGO_VS_AUTO = 0x00  # Vertical swing
    FUEGO_VS_UP = 0x08
    FUEGO_VS_MUP = 0x10
    FUEGO_VS_MIDDLE = 0x18
    FUEGO_VS_MDOWN = 0x20
    FUEGO_VS_DOWN = 0x28
    FUEGO_VS_SWING = 0x38

    op_modes = {
        ACProtocol.MODE_AUTO: FUEGO_MODE_AUTO,
        ACProtocol.MODE_HEAT: FUEGO_MODE_HEAT,
        ACProtocol.MODE_COOL: FUEGO_MODE_COOL,
        ACProtocol.MODE_DRY: FUEGO_MODE_DRY,
        ACProtocol.MODE_FAN: FUEGO_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: FUEGO_FAN_AUTO,
        ACProtocol.FAN_1: FUEGO_FAN1,
        ACProtocol.FAN_2: FUEGO_FAN2,
        ACProtocol.FAN_3: FUEGO_FAN3,
    }
    
    swing_v_map = {
        ACProtocol.VDIR_AUTO: FUEGO_VS_AUTO,
        ACProtocol.VDIR_UP: FUEGO_VS_UP,
        ACProtocol.VDIR_MUP: FUEGO_VS_MUP,
        ACProtocol.VDIR_MIDDLE: FUEGO_VS_MIDDLE,
        ACProtocol.VDIR_MDOWN: FUEGO_VS_MDOWN,
        ACProtocol.VDIR_DOWN: FUEGO_VS_DOWN,
        ACProtocol.VDIR_SWING: FUEGO_VS_SWING,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        power_mode = self.FUEGO_MODE_ON
        temperature = 23
        swing_v = self.FUEGO_VS_SWING

        if power_mode_cmd == self.POWER_OFF:
            power_mode = self.FUEGO_MODE_OFF
            swing_v = self.FUEGO_VS_MUP

        op_mode = self.op_modes.get(operating_mode_cmd, self.FUEGO_MODE_HEAT)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.FUEGO_FAN_AUTO)
        swing_v = self.swing_v_map.get(swing_v_cmd, swing_v)
        if 17 < temperature_cmd < 32:
            temperature = temperature_cmd
        self.send_fuego(power_mode, op_mode, fan_speed, temperature, swing_v)

    def send_fuego(self, power_mode, op_mode, fan_speed, temperature, swing_v):
        send_buffer = bytearray([0x23, 0xCB, 0x26, 0x01, 0x80, 0x20, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00])
                                 # 0     1     2     3     4     5     6     7     8     9    10    11    12    13
        send_buffer[5] = send_buffer[5] | power_mode
        send_buffer[6] = send_buffer[6] | op_mode
        send_buffer[7] = send_buffer[7] | (31 - temperature)
        send_buffer[8] = send_buffer[8] | fan_speed | swing_v
        checksum = 0
        for i in range(0, len(send_buffer) - 1):
            checksum = (checksum + send_buffer[i]) & 0xff
        send_buffer[13] = checksum

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(0, len(send_buffer)):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(0)







from . import ACProtocol


class Daikin2(ACProtocol):
    # Daikin timing constants
    HDR_MARK = 3360  # 3300
    HDR_SPACE = 1760  # 1600
    BIT_MARK = 360  # 310
    ONE_SPACE = 1370  # 1220
    ZERO_SPACE = 520  # 400
    MSG_SPACE = 32300  # 30800

    # Daikin codes
    DAIKIN_MODE_AUTO = 0x00  # Operating mode
    DAIKIN_MODE_HEAT = 0x40
    DAIKIN_MODE_COOL = 0x30
    DAIKIN_MODE_DRY = 0x20
    DAIKIN_MODE_FAN = 0x60
    DAIKIN_POWER_OFF = 0x00  # Power OFF
    DAIKIN_POWER_ON = 0x01
    DAIKIN_FAN_AUTO = 0xA0  # Fan speed
    DAIKIN_FAN1 = 0x30
    DAIKIN_FAN2 = 0x40
    DAIKIN_FAN3 = 0x50
    DAIKIN_FAN4 = 0x60
    DAIKIN_FAN5 = 0x70

    op_modes = {
        ACProtocol.MODE_AUTO: DAIKIN_MODE_AUTO,
        ACProtocol.MODE_HEAT: DAIKIN_MODE_HEAT,
        ACProtocol.MODE_COOL: DAIKIN_MODE_COOL,
        ACProtocol.MODE_DRY: DAIKIN_MODE_DRY,
        ACProtocol.MODE_FAN: DAIKIN_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: DAIKIN_FAN_AUTO,
        ACProtocol.FAN_1: DAIKIN_FAN1,
        ACProtocol.FAN_2: DAIKIN_FAN2,
        ACProtocol.FAN_3: DAIKIN_FAN3,
        ACProtocol.FAN_4: DAIKIN_FAN4,
        ACProtocol.FAN_5: DAIKIN_FAN5,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        power_mode = self.DAIKIN_POWER_ON if power_mode_cmd == self.POWER_ON else self.DAIKIN_POWER_OFF
        op_mode = self.op_modes.get(operating_mode_cmd, self.DAIKIN_MODE_AUTO)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.DAIKIN_FAN_AUTO)
        temperature = 0x10  # 18 deg
        if (operating_mode_cmd == self.MODE_HEAT and 9 < temperature_cmd < 29) \
                or (operating_mode_cmd == self.MODE_COOL and 17 < temperature_cmd < 31):
            temperature = temperature_cmd << 1
        if operating_mode_cmd == self.MODE_DRY:
            temperature = 0x24
        if operating_mode_cmd == self.MODE_FAN:
            temperature = 0xc0

        self.send_daikin(power_mode, op_mode, fan_speed, temperature)

    def send_daikin(self, power_mode, op_mode, fan_speed, temperature):
        send_buffer = bytearray([
            0x11, 0xDA, 0x27, 0x00, 0xC5, 0x00, 0x00, 0xD7,  # First header
            #  0     1     2     3     4     5     6     7
            0x11, 0xDA, 0x27, 0x00, 0x42, 0x49, 0x05, 0xA2,
            # Second header, this seems to have the wall clock time (bytes 12 & 13 are changing)
            #  8     9    10    11    12    13    14    15
            0x11, 0xDA, 0x27, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x60, 0x00, 0x00, 0xC0, 0x00, 0x00, 0x00
            # 16    17    18    19    20    21    22    23    24    25   26     27    28    29    30    31    32    33    34
        ])

        send_buffer[21] = power_mode | op_mode
        send_buffer[22] = temperature
        send_buffer[24] = fan_speed

        checksum = 0

        for i in range(16, 34):
            checksum = (checksum + send_buffer[i]) & 0xff

        send_buffer[34] = checksum

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(0, 8):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(self.MSG_SPACE)
        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(8, 16):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(self.MSG_SPACE)
        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(16, 35):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(0)



from . import ACProtocol


class Carriernqv(ACProtocol):
    #  Carrier (42NQV035G / 38NYV035H2) timing constants (remote control P/N WH-L05SE)
    HDR_MARK = 4320
    HDR_SPACE = 4350
    BIT_MARK = 500
    ONE_SPACE = 1650
    ZERO_SPACE = 550
    MSG_SPACE = 7400

    #  Carrier codes
    CARRIER_MODE_AUTO = 0x00  # Operating mode
    CARRIER_MODE_HEAT = 0xC0
    CARRIER_MODE_COOL = 0x80
    CARRIER_MODE_DRY = 0x40
    CARRIER_MODE_FAN = 0x20
    CARRIER_MODE_OFF = 0xE0  # Power OFF
    CARRIER_FAN_AUTO = 0x00  # Fan speed
    CARRIER_FAN1 = 0x02
    CARRIER_FAN2 = 0x06
    CARRIER_FAN3 = 0x01
    CARRIER_FAN4 = 0x05
    CARRIER_FAN5 = 0x03

    op_modes = {
        ACProtocol.MODE_AUTO: CARRIER_MODE_AUTO,
        ACProtocol.MODE_HEAT: CARRIER_MODE_HEAT,
        ACProtocol.MODE_COOL: CARRIER_MODE_COOL,
        ACProtocol.MODE_DRY: CARRIER_MODE_DRY,
        ACProtocol.MODE_FAN: CARRIER_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: CARRIER_FAN_AUTO,
        ACProtocol.FAN_1: CARRIER_FAN1,
        ACProtocol.FAN_2: CARRIER_FAN2,
        ACProtocol.FAN_3: CARRIER_FAN3,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        op_mode = self.CARRIER_MODE_OFF if power_mode_cmd == self.POWER_OFF else self.op_modes.get(operating_mode_cmd)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.CARRIER_FAN_AUTO)
        temperature = temperature_cmd - 17 if 16 < temperature_cmd < 31 else 7
        self.send_carrier(op_mode, fan_speed, temperature)

    def send_carrier(self, op_mode, fan_speed, temperature):
        send_buffer = bytearray(b'\x4f\xb0\xc0\x3f\x80\x00\x00\x00\x00')
        temperatures = bytearray(b'\x00\x08\x04\x0c\x02\x0a\x06\x0e\x01\x09\x05\x0d\x03\x0b')

        send_buffer[5] = temperatures[temperature]
        send_buffer[6] = op_mode | fan_speed
        checksum = self.calc_checksum(send_buffer)
        send_buffer[8] = self.bit_reverse(checksum)

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for b in send_buffer:
            self.send_byte(b)

        self.mark()
        self.space(self.MSG_SPACE)

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for b in send_buffer:
            self.send_byte(b)

        self.mark()
        self.space(0)

    #  There's something really strange with the checksum calculation...
    #  With these many of the codes matchs with the code from the real Carrier remote
    #  Still certain temperatures do not work with fan speeds 1, 2 or 5
    def calc_checksum(self, send_buffer):
        checksum = 0
        for i in range(8):
            checksum = checksum + self.bit_reverse(send_buffer[i])
        left = send_buffer[6] & 0xf0
        right = send_buffer[6] & 0x0f
        if left == 0:  # MODE_AUTO - certain temperature / fan speed combinations do not work
            checksum = checksum + 0x02
            if right in [0x02, 0x03, 0x06]:
                checksum = checksum + 0x80
        if left == 0x40:  # MODE_DRY - all settings should work
            checksum = checksum + 0x02
        if left == 0xc0 and right in [0x05, 0x06]:  # MODE_HEAT - certain temperature/fan speed combinations do not work
            checksum = checksum + 0xc0
        if left == 0x20:  # MODE_FAN - all settings should work
            checksum = checksum + 0x02
            if right in [0x02, 0x03, 0x06]:
                checksum = checksum + 0x80
        return checksum & 0xff



from . import ACProtocol


class Carriermca(ACProtocol):
    #  Carrier (42MCA009515LS) timing constants (remote control P/N R11CG/E)
    HDR_MARK = 4510
    HDR_SPACE = 4470
    BIT_MARK = 600
    ONE_SPACE = 1560
    ZERO_SPACE = 500

    CARRIER_MODE_AUTO = 0x10  # Operating mode
    CARRIER_MODE_COOL = 0x00
    CARRIER_MODE_DRY = 0x20
    CARRIER_MODE_FAN = 0x20
    CARRIER_MODE_HEAT = 0x30
    CARRIER_MODE_OFF = 0x00  # Power OFF
    CARRIER_MODE_ON = 0x20  # Power ON
    CARRIER_FAN_DRY_AUTO = 0x00  # Fan speed, AUTO or DRY modes
    CARRIER_FAN1 = 0x01
    CARRIER_FAN2 = 0x02
    CARRIER_FAN3 = 0x04
    CARRIER_FAN_AUTO = 0x05
    CARRIER_FAN_OFF = 0x06

    op_modes = {
        ACProtocol.MODE_AUTO: CARRIER_MODE_AUTO,
        ACProtocol.MODE_HEAT: CARRIER_MODE_HEAT,
        ACProtocol.MODE_COOL: CARRIER_MODE_COOL,
        ACProtocol.MODE_DRY: CARRIER_MODE_DRY,
        ACProtocol.MODE_FAN: CARRIER_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: CARRIER_FAN_AUTO,
        ACProtocol.FAN_1: CARRIER_FAN1,
        ACProtocol.FAN_2: CARRIER_FAN2,
        ACProtocol.FAN_3: CARRIER_FAN3,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        power_mode = self.CARRIER_MODE_ON
        temperature = 23

        if power_mode_cmd == self.POWER_OFF:
            power_mode = self.CARRIER_MODE_OFF
            op_mode = self.CARRIER_MODE_COOL
            fan_speed = self.CARRIER_FAN_OFF
            temperature = 31
            self.send_carrier(power_mode, op_mode, fan_speed, temperature)
            return

        op_mode = self.op_modes.get(operating_mode_cmd, self.CARRIER_MODE_COOL)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.CARRIER_FAN_AUTO) if op_mode != self.CARRIER_MODE_FAN \
            else self.CARRIER_FAN_AUTO

        if 16 < temperature_cmd < 31 and op_mode != self.CARRIER_MODE_FAN:
            temperature = temperature_cmd

        self.send_carrier(power_mode, op_mode, fan_speed, temperature)

    def send_carrier(self, power_mode, op_mode, fan_speed, temperature):
        send_buffer = bytearray(b'\x4D\xB2\xD8\x00\x00\x00')
        temperatures = [0, 8, 12, 4, 6, 14, 10, 2, 3, 11, 9, 1, 5, 13, 7]
        send_buffer[2] = send_buffer[2] | power_mode | fan_speed
        send_buffer[4] = send_buffer[4] | op_mode | temperatures[temperature - 17]
        send_buffer[3] = ~send_buffer[2] & 0xff
        send_buffer[5] = ~send_buffer[4] & 0xff

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for b in send_buffer:
            self.send_byte(b)

        self.mark()
        self.space(self.HDR_SPACE)
        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for b in send_buffer:
            self.send_byte(b)

        self.mark()
        self.space(0)



from setuptools import setup

setup(name='hvac-ir',
      version='0.7',
      description='Air conditioner control code generator',
      url='http://github.com/shprota/hvac_ir',
      author='Shprota',
      author_email='shprota@gmail.com',
      license='MIT',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
      ],
      keywords='airconditioner hvac ac control ir infrared heatpump midea gree daikin carrier',
      packages=['hvac_ir'],
      install_requires=[
      ],
      zip_safe=False)



from . import ACProtocol


class Hisense(ACProtocol):
    # Hisense timing constants
    HDR_MARK = 9060
    HDR_SPACE = 4550
    BIT_MARK = 520
    ONE_SPACE = 1700
    ZERO_SPACE = 630
    MSG_SPACE = 8140

    # Power state
    HISENSE_POWER_OFF = 0x04  # **
    HISENSE_POWER_ON = 0x00  # **

    # Operating modes
    # Hisense codes
    HISENSE_MODE_AUTO = 0x04  # Not available 0x00
    HISENSE_MODE_HEAT = 0x00
    HISENSE_MODE_COOL = 0x02
    HISENSE_MODE_DRY = 0x03
    HISENSE_MODE_FAN = 0x04
    HISENSE_MODE_MAINT = 0x04  # Power OFF

    # Fan speeds. Note that some heatpumps have less than 5 fan speeds

    HISENSE_FAN_AUTO = 0x00  # Fan speed
    HISENSE_FAN1 = 0x03  # * low
    HISENSE_FAN2 = 0x02  # * med
    HISENSE_FAN3 = 0x01  # * high

    op_modes = {
        ACProtocol.MODE_AUTO: HISENSE_MODE_AUTO,
        ACProtocol.MODE_HEAT: HISENSE_MODE_HEAT,
        ACProtocol.MODE_COOL: HISENSE_MODE_COOL,
        ACProtocol.MODE_DRY: HISENSE_MODE_DRY,
        ACProtocol.MODE_FAN: HISENSE_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: HISENSE_FAN_AUTO,
        ACProtocol.FAN_1: HISENSE_FAN1,
        ACProtocol.FAN_2: HISENSE_FAN2,
        ACProtocol.FAN_3: HISENSE_FAN3,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        op_mode = self.op_modes.get(operating_mode_cmd, self.HISENSE_MODE_HEAT)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.HISENSE_FAN_AUTO)
        temperature = 23
        if 17 < temperature_cmd < 33:
            temperature = temperature_cmd
        if op_mode == self.HISENSE_MODE_FAN and fan_speed == self.HISENSE_FAN_AUTO:
            fan_speed = self.HISENSE_FAN1
            temperature = 25

        if op_mode == self.HISENSE_MODE_AUTO:
            fan_speed = self.HISENSE_FAN_AUTO

        power_mode = self.HISENSE_POWER_ON if power_mode_cmd == self.POWER_ON else self.HISENSE_POWER_OFF

        self.send_hisense(power_mode, op_mode, fan_speed, temperature)

    def send_hisense(self, power_mode, op_mode, fan_speed, temperature):
        send_buffer = bytearray([0x87, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x00, 0x00, 0x00])
        send_buffer[2] = fan_speed | power_mode
        send_buffer[3] = (((temperature - 18) << 4) | op_mode)
        send_buffer[13] = send_buffer[2]
        for i in range(3, 13):
            send_buffer[13] = send_buffer[13] ^ send_buffer[i]

        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(0, 7):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(self.MSG_SPACE)

        for i in range(7, 14):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(0)



from . import ACProtocol


class Fujitsu(ACProtocol):
    # Fujitsu Nocria (AWYZ14) timing constants (remote control P/N AR-PZ2)
    HDR_MARK = 3210
    HDR_SPACE = 1680
    BIT_MARK = 410
    ONE_SPACE = 1230
    ZERO_SPACE = 440

    # Fujitsu codes
    FUJITSU_MODE_AUTO = 0x00  # Operating mode
    FUJITSU_MODE_HEAT = 0x04
    FUJITSU_MODE_COOL = 0x01
    FUJITSU_MODE_DRY = 0x02
    FUJITSU_MODE_FAN = 0x03
    FUJITSU_MODE_OFF = 0xFF  # Power OFF - not real codes, but we need something...
    FUJITSU_FAN_AUTO = 0x00  # Fan speed
    FUJITSU_FAN1 = 0x04
    FUJITSU_FAN2 = 0x03
    FUJITSU_FAN3 = 0x02
    FUJITSU_FAN4 = 0x01
    FUJITSU_VDIR_MANUAL = 0x00  # Air direction modes
    FUJITSU_VDIR_SWING = 0x10
    FUJITSU_HDIR_MANUAL = 0x00
    FUJITSU_HDIR_SWING = 0x20
    FUJITSU_ECO_OFF = 0x20
    FUJITSU_ECO_ON = 0x00

    op_modes = {
        ACProtocol.MODE_AUTO: FUJITSU_MODE_AUTO,
        ACProtocol.MODE_HEAT: FUJITSU_MODE_HEAT,
        ACProtocol.MODE_COOL: FUJITSU_MODE_COOL,
        ACProtocol.MODE_DRY: FUJITSU_MODE_DRY,
        ACProtocol.MODE_FAN: FUJITSU_MODE_FAN,
    }

    fan_speeds = {
        ACProtocol.FAN_AUTO: FUJITSU_FAN_AUTO,
        ACProtocol.FAN_1: FUJITSU_FAN1,
        ACProtocol.FAN_2: FUJITSU_FAN2,
        ACProtocol.FAN_3: FUJITSU_FAN3,
        ACProtocol.FAN_4: FUJITSU_FAN4,
    }

    swing_v_map = {
        ACProtocol.VDIR_AUTO: FUJITSU_VDIR_MANUAL,
        ACProtocol.VDIR_SWING: FUJITSU_VDIR_SWING,
    }

    swing_h_map = {
        ACProtocol.VDIR_AUTO: FUJITSU_HDIR_MANUAL,
        ACProtocol.VDIR_SWING: FUJITSU_HDIR_SWING,
    }

    def send(self, power_mode_cmd, operating_mode_cmd, fan_speed_cmd, temperature_cmd, swing_v_cmd, swing_h_cmd,
             turbo_cmd=False):
        op_mode = self.op_modes.get(operating_mode_cmd, self.FUJITSU_MODE_HEAT)
        fan_speed = self.fan_speeds.get(fan_speed_cmd, self.FUJITSU_FAN_AUTO)
        swing_v = self.swing_v_map.get(swing_v_cmd, self.FUJITSU_VDIR_MANUAL)
        swing_h = self.swing_v_map.get(swing_h_cmd, self.FUJITSU_HDIR_MANUAL)
        if power_mode_cmd == self.POWER_OFF:
            op_mode = self.FUJITSU_MODE_OFF
        temperature = 23
        if 15 < temperature_cmd < 31:
            temperature = temperature_cmd
        self.send_fujitsu(op_mode, fan_speed, temperature, swing_v, swing_h)

    def send_fujitsu(self, op_mode, fan_speed, temperature, swing_v, swing_h):
        send_buffer = bytearray([0x14, 0x63, 0x00, 0x10, 0x10, 0xFE, 0x09, 0x30, 0x80, 0x04, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00])
        off_command = bytearray([0x14, 0x63, 0x00, 0x10, 0x10, 0x02, 0xFD])
        checksum = 0
        send_buffer[9] = op_mode
        send_buffer[8] = (temperature - 16) << 4 | 0x01
        send_buffer[10] = fan_speed + swing_v + swing_h
        for i in range(0, 15):
            checksum = (checksum + send_buffer[i]) & 0xff
        send_buffer[15] = checksum
        if op_mode == self.FUJITSU_MODE_OFF:
            return self.send_command(off_command)
        return self.send_command(send_buffer)

    def send_command(self, send_buffer):
        self.mark(self.HDR_MARK)
        self.space(self.HDR_SPACE)

        for i in range(0, len(send_buffer)):
            self.send_byte(send_buffer[i])

        self.mark()
        self.space(0)

